@@ -1,9 +1,9 @@
 /*
 *
 * kkstrtext string related and text processing routines
-* $Id: kkstrtext.cc,v 1.33 2003/10/05 15:56:12 konst Exp $
+* $Id: kkstrtext.cc,v 1.38 2004/06/24 08:31:22 konst Exp $
 *
-* Copyright (C) 1999-2002 by Konstantin Klyagin <konst@konst.org.ua>
+* Copyright (C) 1999-2004 by Konstantin Klyagin <konst@konst.org.ua>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -789,7 +789,7 @@ bool iswholeword(const string &s, int so, int eo) {
     return rm && lm;
 }
 
-int hex2int(const string ahex) {
+int hex2int(const string &ahex) {
     int r, i;
 
     r = 0;
@@ -881,14 +881,8 @@ string ruscase(const string &s, const string &mode) {
     return r;
 }
 
-#ifdef ICONV_CONST
-#define ICONV_CCHAR (const char **)
-#else
-#define ICONV_CCHAR
-#endif
-
 string siconv(const string &atext, const string &fromcs, const string &tocs) {
-#ifdef HAVE_ICONV_H
+#ifdef HAVE_ICONV
     iconv_t cd = iconv_open(tocs.c_str(), fromcs.c_str());
 
     if(((int) cd) != -1) {
@@ -903,7 +897,7 @@ string siconv(const string &atext, const string &fromcs, const string &tocs) {
 	    soutleft = outleft = inleft*4;
 	    soutbuf = outbuf = new char[outleft];
 
-	    size_t res = iconv(cd, ICONV_CCHAR &inbuf, &inleft,
+	    size_t res = iconv(cd, (ICONV_CONST char **) &inbuf, &inleft,
 		&outbuf, &outleft);
 
 	    soutbuf[soutleft-outleft] = 0;
@@ -930,8 +924,8 @@ string siconv(const string &atext, const string &fromcs, const string &tocs) {
     return atext;
 }
 
-string cuthtml(const string &html, bool cutbrs) {
-    string r, tag;
+string cuthtml(const string &html, int flags) {
+    string r, tag, buf, token;
     int npos, pos, tpos;
 
     for(pos = 0; (npos = html.find("<", pos)) != -1; pos = npos) {
@@ -943,11 +937,33 @@ string cuthtml(const string &html, bool cutbrs) {
 
 	    tag = html.substr(tpos+1, npos-tpos-2);
 	    if(tag.substr(0, 1) == "/") tag.erase(0, 1);
-
 	    tag = leadcut(trailcut(tag, "/ \n\r"), "/ \n\r");
 
-	    if(tag == "BR" || tag == "br") r += cutbrs ? "\n" : "<br>";
-		else if(cutbrs && (tag == "P" || tag == "p")) r += "\n\n";
+	    buf = ruscase(tag, "toupper");
+	    token = getword(buf);
+
+	    if(token == "BR") r += (flags & chCutBR) ? "\n" : "<br>";
+		else if((flags & chCutBR) && token == "P") r += "\n\n";
+
+	    if(flags & chLeaveLinks) {
+		getword(tag);
+
+		if(token == "A") {
+		    if((tpos = buf.find("HREF")) != -1)
+		    if((tpos = buf.substr(tpos).find("\"")) != -1) {
+			tag.erase(0, tpos+1);
+			r += "[ href: " + getword(tag, "\"") + " ] ";
+		    }
+
+		} else if(token == "IMG") {
+		    if((tpos = buf.find("SRC")) != -1)
+		    if((tpos = buf.substr(tpos).find("\"")) != -1) {
+			tag.erase(0, tpos+1);
+			r += " [ img: " + getword(tag, "\"") + " ]";
+		    }
+
+		}
+	    }
 
 	} else {
 	    r += html.substr(tpos);
@@ -1030,3 +1046,98 @@ char *str_to_utf8(const char *pin) {
     result[i] = '\0';
     return result;
 }
+
+string striprtf(const string &s, const string &charset) {
+    string r, spec, unichar, tmp;
+    char pre = 0;
+    bool bprint, bspec, bunicode;
+    int bparen = -1;
+
+    bprint = true;
+    bspec = bunicode = false;
+
+    for(string::const_iterator i = s.begin(); i != s.end(); ++i) {
+	if(!isalpha(*i) && !isdigit(*i)) bprint = true;
+
+	if(bspec) {
+	    spec += *i;
+
+	    if(spec.size() == 2) {
+		r += (char) hex2int(spec);
+		bspec = false;
+		bprint = true;
+	    }
+
+	} else switch(*i) {
+	    case '{':
+		if(pre != '\\') {
+		    bparen++;
+		    bprint = false;
+		} else {
+		    bprint = true;
+		    r += *i;
+		}
+		break;
+
+	    case '}':
+		if(pre != '\\') {
+		    bprint = false;
+		    bparen--;
+		} else {
+		    bprint = true;
+		    r += *i;
+		}
+
+		break;
+
+	    case '\\':
+		if(pre != '\\') {
+		    bprint = false;
+		} else {
+		    bprint = true;
+		    r += *i;
+		}
+		break;
+
+	    case '\'':
+		if(pre == '\\') {
+		    spec = "";
+		    bspec = true;
+		}
+		break;
+
+	    case 'u':
+		if(pre == '\\') {
+		    unichar = "";
+		    bunicode = true;
+		}
+		break;
+
+	    default:
+		if(!bparen) {
+		    if(bunicode) {
+			unichar += *i;
+
+			if(unichar.size() == 5) {
+			    bunicode = false;
+
+			    if(unichar.substr(0, 4).find_first_not_of("0123456789") == -1) {
+				long l = strtol(unichar.substr(0, 4).c_str(), 0, 0);
+				char ubuf[sizeof(long)+1];
+				memcpy(ubuf, &l, sizeof(long));
+				ubuf[sizeof(long)+1] = 0;
+				r += siconv(ubuf , "utf-16", charset);
+			    }
+			}
+
+		    } else if(bprint) {
+			r += *i;
+		    }
+		}
+	}
+
+	pre = *i;
+    }
+
+    return leadcut(trailcut(r));
+}
