@@ -57,6 +57,7 @@ jconn jab_new(char *user, char *pass, int port, int ssl)
     j->port = port;
 
     j->state = JCONN_STATE_OFF;
+    j->cw_state = 0;
     j->id = 1;
     j->fd = -1;
     j->ssl = ssl;
@@ -127,18 +128,37 @@ void jab_start(jconn j)
     xmlnode x;
     char *t,*t2;
 
-    if(!j || j->state != JCONN_STATE_OFF) return;
+    if(!j || (j->state != JCONN_STATE_OFF && j->state != JCONN_STATE_CONNECTING) ) return;
+    
+    if (!(j->cw_state & CW_CONNECT_WANT_SOMETHING)) { /* same as state != JCONN_STATE_CONNECTING */
+	j->parser = XML_ParserCreate(NULL);
+	XML_SetUserData(j->parser, (void *)j);
+	XML_SetElementHandler(j->parser, startElement, endElement);
+	XML_SetCharacterDataHandler(j->parser, charData);
 
-    j->parser = XML_ParserCreate(NULL);
-    XML_SetUserData(j->parser, (void *)j);
-    XML_SetElementHandler(j->parser, startElement, endElement);
-    XML_SetCharacterDataHandler(j->parser, charData);
-
-    j->fd = make_netsocket(j->port, j->user->server, NETSOCKET_CLIENT, j->ssl);
-    if(j->fd < 0) {
-	STATE_EVT(JCONN_STATE_OFF)
+	if (j->cw_state & CW_CONNECT_BLOCKING)
+	    j->fd = make_netsocket(j->port, j->user->server, NETSOCKET_CLIENT, j->ssl);
+	else
+	    j->fd = make_nb_netsocket(j->port, j->user->server, NETSOCKET_CLIENT, j->ssl, &j->cw_state);
+	
+	if(j->fd < 0) {
+	    STATE_EVT(JCONN_STATE_OFF);
+	    return;
+	}
+    }
+    else { /* subsequent calls to cw_nb_connect until it finishes negociation */
+	if (cw_nb_connect(j->fd, 0, 0, j->ssl, &j->cw_state)) {
+	    STATE_EVT(JCONN_STATE_OFF);
+	    return;
+	}
+    }
+    if (j->cw_state & CW_CONNECT_WANT_SOMETHING){ /* check if it finished negociation */
+	j->state = JCONN_STATE_CONNECTING;
+	STATE_EVT(JCONN_STATE_CONNECTING);
 	return;
     }
+    change_socket_to_blocking(j->fd);
+    
     j->state = JCONN_STATE_CONNECTED;
     STATE_EVT(JCONN_STATE_CONNECTED)
 
@@ -323,6 +343,7 @@ void jab_poll(jconn j, int timeout)
 {
     fd_set fds;
     struct timeval tv;
+    int r;
 
     if (!j || j->state == JCONN_STATE_OFF)
 	return;
@@ -330,17 +351,23 @@ void jab_poll(jconn j, int timeout)
     FD_ZERO(&fds);
     FD_SET(j->fd, &fds);
 
-    if (timeout < 0)
-    {
-	if (select(j->fd + 1, &fds, NULL, NULL, NULL) > 0)
-	    jab_recv(j);
+    if(timeout <= 0) {
+	r = select(j->fd + 1, &fds, NULL, NULL, NULL);
+
+    } else {
+	tv.tv_sec = 0;
+	tv.tv_usec = timeout;
+	r = select(j->fd + 1, &fds, NULL, NULL, &tv);
+
     }
-    else
-    {
-	    tv.tv_sec  = 0;
-	    tv.tv_usec = timeout;
-	    if (select(j->fd + 1, &fds, NULL, NULL, &tv) > 0)
-		jab_recv(j);
+
+    if(r > 0) {
+	jab_recv(j);
+
+    } else if(r) {
+	STATE_EVT(JCONN_STATE_OFF);
+	jab_stop(j);
+
     }
 }
 
