@@ -1,9 +1,9 @@
 /*
 *
 * centericq account manager dialog implementation
-* $Id: accountmanager.cc,v 1.34 2004/02/04 07:44:41 konst Exp $
+* $Id: accountmanager.cc,v 1.38 2004/11/10 11:24:46 konst Exp $
 *
-* Copyright (C) 2001-4 by Konstantin Klyagin <konst@konst.org.ua>
+* Copyright (C) 2001-2004 by Konstantin Klyagin <konst@konst.org.ua>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -75,7 +75,7 @@ void accountmanager::exec() {
     for(fin = false; !fin; ) {
 	t.clear();
 
-	for(pname = icq; pname != protocolname_size; (int) pname += 1) {
+	for(pname = icq; pname != protocolname_size; pname++) {
 	    account = conf.getourid(pname);
 
 	    if(pname != rss)
@@ -126,6 +126,11 @@ void accountmanager::exec() {
 			t.addleaff(n, 0, citem+11, _(" Import friend list : %s "),
 			    stryesno(account.additional["importfriends"] == "1"));
 			break;
+
+		    case irc:
+			t.addleaff(n, 0, citem+12, _(" NickServ password (optional) : %s "),
+			    string(account.additional["nickpass"].size(), '*').c_str());
+			break;
 		}
 
 		if(account.empty()) {
@@ -223,7 +228,13 @@ void accountmanager::exec() {
 
 		case 11:
 		    account.additional["importfriends"] =
-			(account.additional["importfriends"] == "") ? "1" : "";
+			(account.additional["importfriends"] == "") ? "1" : "0";
+		    break;
+
+		case 12:
+		    tmp = face.inputstr(spname + _(" password: "), account.additional["nickpass"], '*');
+		    if(face.getlastinputkey() != KEY_ESC)
+			account.additional["nickpass"] = tmp;
 		    break;
 
 		case 13:
