@@ -1,7 +1,7 @@
 /*
 *
 * centericq core routines
-* $Id: centericq.cc,v 1.187 2004/04/11 16:32:27 konst Exp $
+* $Id: centericq.cc,v 1.193 2004/11/09 23:49:59 konst Exp $
 *
 * Copyright (C) 2001-2003 by Konstantin Klyagin <konst@konst.org.ua>
 *
@@ -71,7 +71,7 @@ void centericq::exec() {
 	if(p)
 	if(rus = (((string) p).substr(0, 2) == "ru")) {
 	    conf.setcharsets("cp1251", "koi8-r");
-	    for(protocolname pname = icq; pname != protocolname_size; (int) pname += 1)
+	    for(protocolname pname = icq; pname != protocolname_size; pname++)
 		conf.setcpconvert(pname, true);
 	}
 
@@ -124,8 +124,8 @@ bool centericq::checkpasswords() {
 
     r = regmode = true;
 
-    for(pname = icq; pname != protocolname_size; (int) pname += 1) {
-	if(!gethook(pname).getCapabs().count(hookcapab::optionalpassword)) {
+    for(pname = icq; pname != protocolname_size; pname++) {
+	if(gethook(pname).enabled() && !gethook(pname).getCapabs().count(hookcapab::optionalpassword)) {
 	    if(!(ia = conf.getourid(pname)).empty()) {
 		if(ia.password.empty()) {
 		    conf.setsavepwd(false);
@@ -152,7 +152,7 @@ bool centericq::checkpasswords() {
 void centericq::inithooks() {
     protocolname pname;
 
-    for(pname = icq; pname != protocolname_size; (int) pname += 1) {
+    for(pname = icq; pname != protocolname_size; pname++) {
 	gethook(pname).init();
     }
 }
@@ -174,6 +174,7 @@ void centericq::mainloop() {
 	    face.getstatkey(key_general_menu, section_contact).c_str(),
 	    face.getstatkey(key_quit, section_contact).c_str());
 
+	face.xtermtitle();
 	c = face.mainloop(action);
 
 	switch(action) {
@@ -452,6 +453,7 @@ void centericq::changestatus() {
 			    conf.setawaymsg(*ipname, tmp);
 
 		    hook.setstatus(st);
+		    reconnect[*ipname].timer = timer_current;
 
 		}
 	    }
@@ -789,8 +791,11 @@ void centericq::handlesignal(int signum) {
 	    cicq.rereadstatus();
 	    break;
 
-	case SIGALRM:
 	case SIGUSR2:
+	    clist.load();
+	    break;
+
+	case SIGALRM:
 	    break;
 
 	case SIGTERM:
@@ -839,7 +844,7 @@ void centericq::rereadstatus() {
     protocolname pname;
     icqconf::imaccount ia;
 
-    for(pname = icq; pname != protocolname_size; (int) pname += 1) {
+    for(pname = icq; pname != protocolname_size; pname++) {
 	ia = conf.getourid(pname);
 
 	if(!ia.empty()) {
@@ -852,7 +857,7 @@ void centericq::rereadstatus() {
 		f >> cst, f.close(), f.clear();
 		unlink(fname.c_str());
 
-		for(st = offline; st != imstatus_size; (int) st += 1) {
+		for(st = offline; st != imstatus_size; st++) {
 		    if(imstatus2char[st] == cst) {
 			gethook(pname).setstatus(st);
 			break;
@@ -1071,6 +1076,7 @@ bool centericq::sendevent(const imevent &ev, icqface::eventviewresult r) {
 	delete sendev;
     }
 
+    face.update();
     return proceed;
 }
 
@@ -1375,7 +1381,7 @@ bool centericq::idle(int options ) {
 	if(!regmode) {
 	    exectimers();
 
-	    for(pname = icq; pname != protocolname_size; (int) pname += 1) {
+	    for(pname = icq; pname != protocolname_size; pname++) {
 		abstracthook &hook = gethook(pname);
 
 		if(hook.online()) {
@@ -1394,7 +1400,7 @@ bool centericq::idle(int options ) {
 	    keypressed = true;
 	    time(&timer_keypress);
 	} else {
-	    for(pname = icq; pname != protocolname_size; (int) pname += 1) {
+	    for(pname = icq; pname != protocolname_size; pname++) {
 		abstracthook &hook = gethook(pname);
 
 		if(hook.online())
@@ -1424,7 +1430,7 @@ void centericq::setauto(imstatus astatus) {
 	nautoset = false;
     }
 
-    for(pname = icq; pname != protocolname_size; (int) pname += 1) {
+    for(pname = icq; pname != protocolname_size; pname++) {
 	abstracthook &hook = gethook(pname);
 	stcurrent = hook.getstatus();
 
@@ -1481,7 +1487,7 @@ void centericq::exectimers() {
     *
     */
 
-    for(pname = icq; pname != protocolname_size; (int) pname += 1) {
+    for(pname = icq; pname != protocolname_size; pname++) {
 	if(!conf.getourid(pname).empty() || (pname == rss)) {
 	    abstracthook &hook = gethook(pname);
 
@@ -1493,8 +1499,6 @@ void centericq::exectimers() {
 
 	    hook.exectimers();
 
-	    static map<protocolname, reconnectInfo> reconnect;
-
 	    if(timer_current-reconnect[pname].timer > reconnect[pname].period) {
 		/*
 		*
@@ -1503,7 +1507,7 @@ void centericq::exectimers() {
 		*/
 
 		if(!hook.logged()) {
-		    time(&reconnect[pname].timer);
+		    reconnect[pname].timer = timer_current;
 
 		    if(reconnect[pname].period < 180)
 			reconnect[pname].period += reconnect[pname].period/2;
@@ -1577,7 +1581,7 @@ void centericq::exectimers() {
 
 	if(!conf.enoughdiskspace()) {
 	    if(fonline) {
-		for(pname = icq; pname != protocolname_size; (int) pname += 1)
+		for(pname = icq; pname != protocolname_size; pname++)
 		    gethook(pname).disconnect();
 
 		face.log(_("! free disk space is less than 10k, going offline"));
@@ -1645,8 +1649,8 @@ void centericq::massmove() {
     icqcontact *c;
 
     face.muins.clear();
-    face.multicontacts(_("Select contacts to move"));
 
+    if(face.multicontacts(_("Select contacts to move")))
     if(!face.muins.empty())
     if(gid = face.selectgroup(_("Mass move selected users to.."))) {
 	for(im = face.muins.begin(); im != face.muins.end(); ++im) {
