@@ -1,7 +1,7 @@
 /*
 *
 * centericq gadu-gadu protocol handling class
-* $Id: gaduhook.cc,v 1.6 2004/04/11 16:32:28 konst Exp $
+* $Id: gaduhook.cc,v 1.12 2004/08/05 18:39:34 konst Exp $
 *
 * Copyright (C) 2004 by Konstantin Klyagin <konst@konst.org.ua>
 *
@@ -46,38 +46,55 @@ extern "C" {
 
 #define PERIOD_PING  50
 
-imstatus gg2imstatus(int st) {
+static imstatus gg2imstatus(int st) {
     imstatus imst;
 
     switch(st) {
-	case GG_STATUS_INVISIBLE: imst = invisible; break;
-	case GG_STATUS_BUSY: imst = occupied; break;
-	case GG_STATUS_NOT_AVAIL: imst = notavail; break;
-	default: imst = available; break;
+	case GG_STATUS_INVISIBLE:
+	case GG_STATUS_INVISIBLE_DESCR:
+	    imst = invisible;
+	    break;
+
+	case GG_STATUS_BUSY:
+	case GG_STATUS_BUSY_DESCR:
+	    imst = occupied;
+	    break;
+
+	case GG_STATUS_NOT_AVAIL:
+	case GG_STATUS_NOT_AVAIL_DESCR:
+	    imst = offline;
+	    break;
+
+	default:
+	    imst = available;
+	    break;
     }
 
     return imst;
 }
 
-int imstatus2gg(imstatus st) {
+static int imstatus2gg(imstatus st, const string &desc = "") {
     int gst;
 
     switch(st) {
 	case invisible:
-	    gst = GG_STATUS_INVISIBLE;
+	    gst = desc.empty() ?
+		GG_STATUS_INVISIBLE : GG_STATUS_INVISIBLE_DESCR;
 	    break;
+
 	case dontdisturb:
 	case occupied:
-	    gst = GG_STATUS_BUSY;
-	    break;
 	case notavail:
 	case away:
-	    gst = GG_STATUS_NOT_AVAIL;
+	    gst = desc.empty() ?
+		GG_STATUS_BUSY : GG_STATUS_BUSY_DESCR;
 	    break;
+
 	case freeforchat:
 	case available:
 	default:
-	    gst = GG_STATUS_AVAIL;
+	    gst = desc.empty() ?
+		GG_STATUS_AVAIL : GG_STATUS_AVAIL_DESCR;
 	    break;
     }
 
@@ -94,6 +111,8 @@ gaduhook::gaduhook(): abstracthook(gadu), flogged(false), sess(0) {
     fcapabs.insert(hookcapab::changepassword);
     fcapabs.insert(hookcapab::changenick);
     fcapabs.insert(hookcapab::changedetails);
+    fcapabs.insert(hookcapab::setaway);
+    fcapabs.insert(hookcapab::fetchaway);
     fcapabs.insert(hookcapab::ssl);
 }
 
@@ -122,10 +141,19 @@ void gaduhook::connect() {
 	lp.uin = acc.uin;
 	lp.password = pass.get();
 	lp.async = 1;
-	lp.status = imstatus2gg(manualstatus);
 
-	sess = gg_login(&lp);
+	static auto_ptr<char> descr(strdup(rusconv("kw", conf.getawaymsg(proto)).c_str()));
+
+	lp.status_descr = descr.get();
+	lp.status = imstatus2gg(manualstatus, descr.get());
+
+	lp.tls = acc.additional["ssl"] == "1" ? 1 : 0;
+
 	log(logConnecting);
+	sess = gg_login(&lp);
+
+	if(!sess)
+	    face.log(_("+ [gg] connection failed"));
 
     } else {
 	face.log(_("+ [gg] cannot resolve %s"), acc.server.c_str());
@@ -135,10 +163,12 @@ void gaduhook::connect() {
 
 void gaduhook::cutoff() {
     if(sess) {
+	gg_change_status(sess, GG_STATUS_NOT_AVAIL);
 	gg_logoff(sess);
 	gg_free_session(sess);
 	sess = 0;
     }
+
     clist.setoffline(proto);
 }
 
@@ -219,11 +249,11 @@ void gaduhook::main() {
 		break;
 
 	    case GG_EVENT_STATUS:
-		userstatuschange(e->event.status.uin, e->event.status.status);
+		userstatuschange(e->event.status.uin, e->event.status.status, e->event.status.descr);
 		break;
 
 	    case GG_EVENT_STATUS60:
-		userstatuschange(e->event.status60.uin, e->event.status60.status);
+		userstatuschange(e->event.status60.uin, e->event.status60.status, e->event.status60.descr);
 		break;
 
 	    case GG_EVENT_ACK:
@@ -331,7 +361,7 @@ void gaduhook::setautostatus(imstatus st) {
 
     } else {
 	if(getstatus() != offline) {
-	    gg_change_status(sess, imstatus2gg(st));
+	    gg_change_status_descr(sess, imstatus2gg(st, conf.getawaymsg(proto)), conf.getawaymsg(proto).c_str());
 	    logger.putourstatus(proto, getstatus(), st);
 	} else {
 	    connect();
@@ -343,10 +373,10 @@ void gaduhook::setautostatus(imstatus st) {
 imstatus gaduhook::getstatus() const {
     if(!sess) return offline;
 
-    if(GG_S_A(sess->status)) return available; else
     if(GG_S_NA(sess->status)) return notavail; else
     if(GG_S_B(sess->status)) return occupied; else
-    if(GG_S_I(sess->status)) return invisible;
+    if(GG_S_I(sess->status)) return invisible; else
+	return available;
 }
 
 void gaduhook::requestinfo(const imcontact &c) {
@@ -357,7 +387,19 @@ void gaduhook::requestinfo(const imcontact &c) {
     gg_pubdir50(sess, ireq);
 }
 
-void gaduhook::requestawaymsg(const imcontact &c) {
+void gaduhook::requestawaymsg(const imcontact &ic) {
+    icqcontact *c = clist.get(ic);
+
+    if(c) {
+	string am = awaymsgs[ic.uin];
+
+	if(!am.empty()) {
+	    em.store(imnotification(ic, (string) _("Away message:") + "\n\n" + am));
+	} else {
+	    face.log(_("+ [gg] no away message from %s, %s"),
+		c->getdispnick().c_str(), ic.totext().c_str());
+	}
+    }
 }
 
 bool gaduhook::regconnect(const string &aserv) {
@@ -530,10 +572,17 @@ void gaduhook::searchdone(void *p) {
     }
 }
 
-void gaduhook::userstatuschange(unsigned int uin, int status) {
+void gaduhook::userstatuschange(unsigned int uin, int status, const char *desc) {
     icqcontact *c = clist.get(imcontact(uin, gadu));
     if(c) {
-	c->setstatus(gg2imstatus(status));
+	imstatus ust = gg2imstatus(status);
+	logger.putonline(c, c->getstatus(), ust);
+	c->setstatus(ust);
+	
+	if(desc)
+	    awaymsgs[uin] = rusconv("wk", desc);
+	else
+	    awaymsgs[uin] = "";
     }
 }
 
@@ -571,10 +620,18 @@ unsigned int ip, int port, int version) {
 	char *p = inet_ntoa(addr);
 	if(p) c->setlastip(p);
 
-	c->setstatus(gg2imstatus(status));
+	imstatus ust = gg2imstatus(status);
+	logger.putonline(c, c->getstatus(), ust);
+	c->setstatus(ust);
+
+	if(desc)
+	    awaymsgs[ic.uin] = rusconv("wk", desc);
+	else
+	    awaymsgs[ic.uin] = "";
     }
 }
 
+
 // ----------------------------------------------------------------------------
 
 const int token_char_height = 12; 
