@@ -1,7 +1,7 @@
 /*
 *
 * centericq icq protocol handling class
-* $Id: icqhook.cc,v 1.150 2004/04/11 16:32:28 konst Exp $
+* $Id: icqhook.cc,v 1.158 2004/11/11 13:42:05 konst Exp $
 *
 * Copyright (C) 2001-2004 by Konstantin Klyagin <konst@konst.org.ua>
 *
@@ -31,6 +31,8 @@
 #include "accountmanager.h"
 #include "eventmanager.h"
 
+#include "src/Capabilities.h"
+
 #define PERIOD_ICQPOLL  5
 #define PERIOD_RESOLVE  10
 #define DELAY_SENDNEW   5
@@ -84,10 +86,6 @@ icqhook::icqhook(): abstracthook(icq) {
     cli.self_contact_userinfo_change_signal.connect(this, &icqhook::self_contact_userinfo_change_cb);
     cli.self_contact_status_change_signal.connect(this, &icqhook::self_contact_status_change_cb);
     cli.sbl_received.connect(this, &icqhook::sbl_received_cb);
-
-#ifdef DEBUG
-    cli.logger.connect(this, &icqhook::logger_cb);
-#endif
 }
 
 icqhook::~icqhook() {
@@ -111,6 +109,9 @@ icqhook::~icqhook() {
 
 void icqhook::init() {
     manualstatus = conf.getstatus(proto);
+
+    if(conf.getdebug())
+	cli.logger.connect(this, &icqhook::logger_cb);
 }
 
 void icqhook::connect() {
@@ -995,7 +996,7 @@ void icqhook::renamegroup(const string &oldname, const string &newname) {
 void icqhook::connected_cb(ConnectedEvent *ev) {
     flogged = true;
 
-    time(&timer_poll);
+    timer_poll = timer_current;
     timer_resolve = timer_poll-PERIOD_RESOLVE+3;
 
     logger.putourstatus(icq, offline, manualstatus);
@@ -1034,7 +1035,7 @@ void icqhook::disconnected_cb(DisconnectedEvent *ev) {
 	_("turboing"),
 	_("bad password"),
 	_("username and password mismatch"),
-	_("already logged"),
+	_("dual login detected"),
 	""
     };
 
@@ -1042,34 +1043,74 @@ void icqhook::disconnected_cb(DisconnectedEvent *ev) {
     wfds.clear();
     efds.clear();
 
+    msg = isconnecting() ? _("+ [icq] cannot connect") : _("+ [icq] disconnected");
+
     logger.putourstatus(icq, getstatus(), offline);
 
     clist.setoffline(icq);
     fonline = false;
 
-    msg = _("+ [icq] disconnected");
-
     if(!reasons[ev->getReason()].empty()) {
-	msg += ", ";
+	msg += ": ";
 	msg += reasons[ev->getReason()];
 
 	logger.putmessage((string) _("icq disconnection reason") + ": " +
 	    reasons[ev->getReason()]);
+
+	if(ev->getReason() == DisconnectedEvent::FAILED_DUALLOGIN)
+	    manualstatus = offline;
     }
 
     face.log(msg);
     face.update();
 }
 
+static string fixicqrtf(string msg) {
+    int pos, bpos, n;
+    string sub;
+
+    static char *emoticons[] = {
+	":-)", ":-O", ":-|", ":-\\", ":-(", ":-*", "8-/", ":~(",
+	";-)", ">:-O", ":`(", ":-X", ":-P", "B-)", "O:-)", ":-D",
+	"*ANNOYED*", "*DISGUSTED*", "*DROOLING*", "*GIGGLING*",
+	"*JOKINGLY*", "*SHOCKED*", "*WHINNING*", "*SURPRISED*",
+	"*SURPRISED*", "*IN LOVE*"
+    };
+
+    for(pos = 0; (pos = msg.find("<##icqimage", pos)) != -1; ) {
+	if((bpos = msg.find(">", pos)) != -1) {
+	    sub = msg.substr(pos, bpos-pos);
+
+	    if(sub.size() > 2) sub.erase(0, sub.size()-2);
+	    n = hex2int(sub);
+
+	    if(n >= 0 && n <= 25) {
+		msg.erase(pos, bpos-pos+1);
+		msg.insert(pos, emoticons[n]);
+	    } else {
+		pos++;
+	    }
+	}
+    }
+
+    return msg;
+}
+
 void icqhook::messaged_cb(MessageEvent *ev) {
     imcontact ic;
     time_t t;
+    string text;
 
     ic = imcontact(ev->getContact()->getUIN(), icq);
 
     if(ev->getType() == MessageEvent::Normal) {
 	NormalMessageEvent *r = static_cast<NormalMessageEvent *>(ev);
-	em.store(immessage(ic, imevent::incoming, rusconv("wk", r->getMessage()), r->getTime()));
+
+	text = r->getMessage();
+	if(text.substr(0, 6) == "{\\rtf1")
+	    text = fixicqrtf(striprtf(text, conf.getconvertfrom(proto)));
+
+	em.store(immessage(ic, imevent::incoming, rusconv("wk", text), r->getTime()));
 
     } else if(ev->getType() == MessageEvent::URL) {
 	URLMessageEvent *r = static_cast<URLMessageEvent *>(ev);
@@ -1279,9 +1320,7 @@ void icqhook::logger_cb(LogEvent *ev) {
     switch(ev->getType()) {
 	case LogEvent::PACKET:
 	case LogEvent::DIRECTPACKET:
-#if PACKETDEBUG
 	    face.log(ev->getMessage());
-#endif
 	    break;
 
 	default:
