@@ -1,7 +1,7 @@
 /*
 *
 * centericq Jabber protocol handling class
-* $Id: jabberhook.cc,v 1.70 2004/04/13 10:37:50 konst Exp $
+* $Id: jabberhook.cc,v 1.76 2004/11/11 13:42:05 konst Exp $
 *
 * Copyright (C) 2002-2005 by Konstantin Klyagin <konst@konst.org.ua>
 *
@@ -53,11 +53,9 @@ static void jidsplit(const string &jid, string &user, string &host, string &rest
 static string jidtodisp(const string &jid) {
     string user, host, rest;
     jidsplit(jid, user, host, rest);
-/*
-    if(host != "jabber.com" && !host.empty())
+
+    if(!host.empty())
 	user += (string) "@" + host;
-*/
-    user += (string) "@" + host;
 
     return user;
 }
@@ -72,7 +70,10 @@ string jabberhook::jidnormalize(const string &jid) const {
 
     string user, host, rest;
     jidsplit(jid, user, host, rest);
-    if(host.empty()) host = "jabber.com";
+
+    if(host.empty())
+	host = conf.getourid(proto).server;
+
     user += (string) "@" + host;
     if(!rest.empty()) user += (string) "/" + rest;
     return user;
@@ -121,9 +122,8 @@ void jabberhook::connect() {
     jab_packet_handler(jc, &packethandler);
     jab_state_handler(jc, &statehandler);
 
-#if PACKETDEBUG
-    jab_logger(jc, &jlogger);
-#endif
+    if(conf.getdebug())
+	jab_logger(jc, &jlogger);
 
     if(jc->user) {
 	fonline = true;
@@ -150,6 +150,11 @@ void jabberhook::main() {
     xmlnode x, z;
     char *cid;
 
+    if(jc && jc->state == JCONN_STATE_CONNECTING) {
+	jab_start(jc);
+	return;
+    }
+    
     jab_poll(jc, 0);
 
     if(jstate == STATE_CONNECTING) {
@@ -178,18 +183,24 @@ void jabberhook::main() {
 
     } else if(jc->state == JCONN_STATE_OFF || jc->fd == -1) {
 	statehandler(jc, JCONN_STATE_OFF);
+
     }
 }
 
 void jabberhook::getsockets(fd_set &rfds, fd_set &wfds, fd_set &efds, int &hsocket) const {
     if(jc) {
-	FD_SET(jc->fd, &rfds);
+	if (jc->cw_state & CW_CONNECT_WANT_READ)
+	    FD_SET(jc->fd, &rfds);
+	else if (jc->cw_state & CW_CONNECT_WANT_WRITE)
+	    FD_SET(jc->fd, &wfds);
+	else
+	    FD_SET(jc->fd, &rfds);
 	hsocket = max(jc->fd, hsocket);
     }
 }
 
 bool jabberhook::isoursocket(fd_set &rfds, fd_set &wfds, fd_set &efds) const {
-    if(jc) return FD_ISSET(jc->fd, &rfds);
+    if(jc) return FD_ISSET(jc->fd, &rfds) || FD_ISSET(jc->fd, &wfds);
     return false;
 }
 
@@ -508,12 +519,13 @@ const string &serv, string &err) {
     jab_packet_handler(jc, &packethandler);
     jab_state_handler(jc, &statehandler);
 
-#if PACKETDEBUG
-    jab_logger(jc, &jlogger);
-#endif
+    if(conf.getdebug())
+	jab_logger(jc, &jlogger);
 
+    jc->cw_state = CW_CONNECT_BLOCKING;
     jab_start(jc);
-    id = atoi(jab_reg(jc));
+
+    if(jc) id = atoi(jab_reg(jc));
 
     if(!online()) {
 	err = _("Unable to connect");
@@ -1252,10 +1264,11 @@ void jabberhook::statehandler(jconn conn, int state) {
 
     switch(state) {
 	case JCONN_STATE_OFF:
+	    jhook.flogged = jhook.fonline = false;
+
 	    if(previous_state != JCONN_STATE_OFF) {
 		logger.putourstatus(jhook.proto, jhook.getstatus(), jhook.ourstatus = offline);
 		jhook.log(logDisconnected);
-		jhook.flogged = jhook.fonline = false;
 		jhook.roster.clear();
 		jhook.agents.clear();
 		clist.setoffline(jhook.proto);
