@@ -1,7 +1,7 @@
 /*
 *
 * centericq yahoo! protocol handling class
-* $Id: yahoohook.cc,v 1.106 2004/03/22 21:10:18 konst Exp $
+* $Id: yahoohook.cc,v 1.111 2004/07/20 06:51:05 konst Exp $
 *
 * Copyright (C) 2003-2004 by Konstantin Klyagin <konst@konst.org.ua>
 *
@@ -38,6 +38,7 @@
 #include "eventmanager.h"
 
 #include "yahoo2.h"
+#include "connwrap.h"
 
 #include <netdb.h>
 #include <sys/socket.h>
@@ -134,18 +135,6 @@ void yahoohook::connect() {
     icqconf::imaccount acc = conf.getourid(proto);
     int r;
 
-    strcpy(pager_host, acc.server.c_str());
-    strcpy(pager_port, i2str(acc.port).c_str());
-
-    strcpy(filetransfer_host, "filetransfer.msg.yahoo.com");
-    strcpy(filetransfer_port, "80");
-
-    strcpy(webcam_host, "webcam.yahoo.com");
-    strcpy(webcam_port, "5100");
-    strcpy(webcam_description, "Philips ToUcam Pro");
-    strcpy(local_host, "");
-    conn_type = 1;
-
     log(logConnecting);
 
     if(cid < 0) {
@@ -153,7 +142,10 @@ void yahoohook::connect() {
 	yahoo_close(cid);
     }
 
-    cid = yahoo_init(acc.nickname.c_str(), acc.password.c_str());
+    cid = yahoo_init_with_attributes(acc.nickname.c_str(), acc.password.c_str(),
+	"pager_host", acc.server.c_str(),
+	"pager_port", acc.port, 0);
+
     yahoo_login(cid, stat2int[manualstatus]);
 
     if(cid < 0) {
@@ -255,6 +247,9 @@ void yahoohook::disconnect() {
 
 void yahoohook::disconnected() {
     if(logged()) {
+	rfds.clear();
+	wfds.clear();
+
 	logger.putourstatus(proto, getstatus(), ourstatus = offline);
 	clist.setoffline(proto);
 	fonline = false;
@@ -301,11 +296,11 @@ bool yahoohook::send(const imevent &ev) {
     if(c) {
 	if(ev.gettype() == imevent::message) {
 	    const immessage *m = static_cast<const immessage *>(&ev);
-	    if(m) text = rushtmlconv("kw", m->gettext());
+	    if(m) text = rushtmlconv("ku", m->gettext());
 
 	} else if(ev.gettype() == imevent::url) {
 	    const imurl *m = static_cast<const imurl *>(&ev);
-	    if(m) text = rushtmlconv("kw", m->geturl()) + "\n\n" + rusconv("kw", m->getdescription());
+	    if(m) text = rushtmlconv("ku", m->geturl()) + "\n\n" + rusconv("kw", m->getdescription());
 
 	} else if(ev.gettype() == imevent::file) {
 	    const imfile *m = static_cast<const imfile *>(&ev);
@@ -367,7 +362,7 @@ void yahoohook::sendnewuser(const imcontact &ic, bool report) {
 		if(report) log(logContactAdd, ic.nickname.c_str());
 		vector<icqgroup>::const_iterator ig = find(groups.begin(), groups.end(), clist.get(ic)->getgroupid());
 		if(ig != groups.end()) {
-		    yahoo_add_buddy(cid, ic.nickname.c_str(), ig->getname().c_str());
+		    yahoo_add_buddy(cid, ic.nickname.c_str(), ig->getname().c_str(), "");
 		}
 	    }
 	}
@@ -465,7 +460,7 @@ void yahoohook::setautostatus(imstatus st) {
 		yahoo_set_away(cid, (yahoo_status) stat2int[st], 0, 0);
 
 	    } else {
-		auto_ptr<char> msg(strdup(rusconv("kw", conf.getawaymsg(proto)).c_str()));
+		auto_ptr<char> msg(strdup(rusconv("ku", conf.getawaymsg(proto)).c_str()));
 		yahoo_set_away(cid, (yahoo_status) stat2int[st], msg.get(), 1);
 
 	    }
@@ -551,7 +546,9 @@ bool yahoohook::knowntransfer(const imfile &fr) const {
 
 void yahoohook::replytransfer(const imfile &fr, bool accept, const string &localpath) {
     if(accept) {
-	sfiles.push_back(strdup(localpath.c_str()));
+	string localname = localpath + "/";
+	localname += fr.getfiles()[0].fname;
+	sfiles.push_back(strdup(localname.c_str()));
 	srfiles[sfiles.back()] = fr;
 	yahoo_get_url_handle(cid, fvalid[fr].c_str(), &get_url, sfiles.back());
 
@@ -733,6 +730,7 @@ void yahoohook::login_response(int id, int succ, char *url) {
 	case YAHOO_LOGIN_DUPL:
 	    face.log(_("+ [yahoo] another logon detected"));
 	    yahoo_logoff(yhook.cid);
+	    yhook.manualstatus = offline;
 	    break;
 
 	case YAHOO_LOGIN_SOCK:
@@ -760,7 +758,7 @@ void yahoohook::status_changed(int id, char *who, int stat, char *msg, int away)
     yhook.userstatus(who, stat, msg ? msg : "", (bool) away);
 }
 
-void yahoohook::got_im(int id, char *who, char *msg, long tm, int stat, int utf8) {
+void yahoohook::got_im(int id, char *me, char *who, char *msg, long tm, int stat, int utf8) {
     imcontact ic(who, yahoo);
     string text = cuthtml(msg, chCutBR | chLeaveLinks);
 
@@ -935,13 +933,24 @@ void yahoohook::conf_message(int id, char *who, char *room, char *msg, int utf8)
     if(c) em.store(immessage(c, imevent::incoming, text));
 }
 
-void yahoohook::got_file(int id, char *who, char *url, long expires, char *msg, char *fname, unsigned long fesize) {
-    if(!who || !url || !msg || !fname)
+void yahoohook::got_file(int id, char *me, char *who, char *url, long expires, char *msg, char *fname, unsigned long fesize) {
+    if(!who || !url)
 	return;
 
+    if(!fname) {
+	fname = strrchr(url, '/');
+	if(fname) fname++;
+	    else return;
+    }
+
+    int pos;
     imfile::record r;
     r.fname = fname;
     r.size = fesize;
+    if(!fesize) r.size = -1;
+
+    if((pos = r.fname.find('?')) != -1)
+	r.fname.erase(pos);
 
     imfile fr(imcontact(who, yahoo), imevent::incoming, "",
 	vector<imfile::record>(1, r));
@@ -963,12 +972,12 @@ void yahoohook::contact_added(int id, char *myid, char *who, char *msg) {
     em.store(imnotification(imcontact(who, yahoo), text));
 }
 
-void yahoohook::typing_notify(int id, char *who, int stat) {
+void yahoohook::typing_notify(int id, char *me, char *who, int stat) {
     icqcontact *c = clist.get(imcontact(who, yahoo));
     if(c) c->setlasttyping(stat ? timer_current : 0);
 }
 
-void yahoohook::game_notify(int id, char *who, int stat) {
+void yahoohook::game_notify(int id, char *me, char *who, int stat) {
 }
 
 void yahoohook::mail_notify(int id, char *from, char *subj, int cnt) {
@@ -985,7 +994,7 @@ void yahoohook::system_message(int id, char *msg) {
     face.log(_("+ [yahoo] system: %s"), msg);
 }
 
-void yahoohook::error(int id, char *err, int fatal) {
+void yahoohook::error(int id, char *err, int fatal, int num) {
     if(fatal) {
 	face.log(_("+ [yahoo] error: %s"), err);
 	yhook.disconnected();
@@ -1009,7 +1018,7 @@ int yahoohook::add_handler(int id, int fd, yahoo_input_condition cond, void *dat
     return tag;
 }
 
-void yahoohook::remove_handler(int tag) {
+void yahoohook::remove_handler(int id, int tag) {
     vector<yfd>::iterator i;
 
     i = find(yhook.rfds.begin(), yhook.rfds.end(), tag);
@@ -1045,7 +1054,7 @@ int yahoohook::connect_async(int id, char *host, int port, yahoo_connect_callbac
     int f = fcntl(servfd, F_GETFL);
     fcntl(servfd, F_SETFL, f | O_NONBLOCK);
 
-    error = ::connect(servfd, (struct sockaddr *) &serv_addr, sizeof(serv_addr));
+    error = cw_connect(servfd, (struct sockaddr *) &serv_addr, sizeof(serv_addr), 0);
 
     if(!error) {
 	callback(servfd, 0, data);
@@ -1067,19 +1076,19 @@ int yahoohook::connect_async(int id, char *host, int port, yahoo_connect_callbac
 
 void yahoohook::connect_complete(void *data, int source) {
     connect_callback_data *ccd = (connect_callback_data *) data;
-    char error;
-    socklen_t err_size = sizeof(error);
+    int so_error;
+    socklen_t err_size = sizeof(so_error);
 
-    remove_handler(ccd->tag);
+    remove_handler(0, ccd->tag);
 
-    if(getsockopt(source, SOL_SOCKET, SO_ERROR, &error, &err_size) == -1 || error != 0) {
+    if(getsockopt(source, SOL_SOCKET, SO_ERROR, (char *) &so_error, &err_size) == -1 || so_error != 0) {
 	if(yhook.logged())
 	    face.log(_("+ [yahoo] direct connection failed"));
 
 	close(source);
 	source = -1;
     } else {
-	ccd->callback(source, error, ccd->callback_data);
+	ccd->callback(source, so_error, ccd->callback_data);
     }
 
     free(ccd);
@@ -1112,10 +1121,10 @@ void yahoohook::rejected(int id, char *who, char *msg) {
 void yahoohook::got_webcam_image(int id, const char * who, const unsigned char *image, unsigned int image_size, unsigned int real_size, unsigned int timestamp) {
 }
 
-void yahoohook::webcam_invite(int id, char *from) {
+void yahoohook::webcam_invite(int id, char *me, char *from) {
 }
 
-void yahoohook::webcam_invite_reply(int id, char *from, int accept) {
+void yahoohook::webcam_invite_reply(int id, char *me, char *from, int accept) {
 }
 
 void yahoohook::webcam_closed(int id, char *who, int reason) {
@@ -1128,6 +1137,17 @@ void yahoohook::webcam_data_request(int id, int send) {
 }
 
 int yahoohook::ylog(char *fmt, ...) {
+    if(conf.getdebug()) {
+	char buf[512];
+	va_list ap;
+
+	va_start(ap, fmt);
+	vsprintf(buf, fmt, ap);
+	va_end(ap);
+
+	face.log(buf);
+    }
+
     return 0;
 }
 
@@ -1167,12 +1187,12 @@ void yahoohook::get_fd(int id, int fd, int error, void *data) {
 
 void yahoohook::get_url(int id, int fd, int error, const char *filename, unsigned long size, void *data) {
     int rsize = 0;
-    const char *localdir = (const char *) data;
+    const char *localname = (const char *) data;
 
     if(!error) {
-	vector<char *>::iterator i = find(yhook.sfiles.begin(), yhook.sfiles.end(), localdir);
+	vector<char *>::iterator i = find(yhook.sfiles.begin(), yhook.sfiles.end(), localname);
 	if(i != yhook.sfiles.end()) {
-	    ofstream f(((string) localdir + "/" + filename).c_str());
+	    ofstream f(localname);
 
 	    if(f.is_open()) {
 		int r;
@@ -1191,13 +1211,14 @@ void yahoohook::get_url(int id, int fd, int error, const char *filename, unsigne
 		error = -1;
 	    }
 
+	    delete *i;
 	    yhook.sfiles.erase(i);
 	}
     }
 
-    map<const char *, imfile>::iterator ir = yhook.srfiles.find(localdir);
+    map<const char *, imfile>::iterator ir = yhook.srfiles.find(localname);
     if(ir != yhook.srfiles.end()) {
-	face.transferupdate(filename, ir->second,
+	face.transferupdate(justfname(localname), ir->second,
 	    error ? icqface::tsError : icqface::tsFinish,
 	    size, rsize);
 
