@@ -69,8 +69,6 @@ using namespace std;
 #define PERIOD_ATONCE           5
 #define MAX_ATONCE              10
 
-#define DEFAULT_CHARSET "ISO-8859-1"
-
 /*
 *
 * Several helper routines
@@ -80,4 +78,13 @@ using namespace std;
 string up(string s);
 string lo(string s);
 
+/*! .. and defines.
+ */
+
+#define ENUM_PLUSPLUS(tp) \
+    inline tp& operator ++(tp &p, int) { \
+	int t = p; \
+	return p = static_cast<tp>(++t); \
+    }
+
 #endif
