@@ -1,7 +1,7 @@
 /*
 *
 * centericq configuration handling routines
-* $Id: icqconf.cc,v 1.130 2004/03/17 19:08:31 konst Exp $
+* $Id: icqconf.cc,v 1.137 2004/11/11 13:42:05 konst Exp $
 *
 * Copyright (C) 2001-2004 by Konstantin Klyagin <konst@konst.org.ua>
 *
@@ -48,11 +48,12 @@ icqconf::icqconf() {
     autoaway = autona = 0;
 
     hideoffline = antispam = makelog = askaway = logtimestamps =
-	logonline = emacs = false;
+	logonline = emacs = proxyssl = proxyconnect = notitles =
+	debug = false;
 
     savepwd = mailcheck = fenoughdiskspace = true;
 
-    for(protocolname pname = icq; pname != protocolname_size; (int) pname += 1) {
+    for(protocolname pname = icq; pname != protocolname_size; pname++) {
 	chatmode[pname] = true;
 	cpconvert[pname] = entersends[pname] = nonimonline[pname] = false;
     }
@@ -267,9 +268,9 @@ void icqconf::loadkeys() {
 	    k.ctrl = false;
 	    k.alt = false;
 
-	    param = getword(buf);	/* first word is always bind */
+	    param = getword(buf);       /* first word is always bind */
 
-	    param = getword(buf);	/* read section */
+	    param = getword(buf);       /* read section */
 	    if(param == "contact")
 		k.section = section_contact; else
 	    if(param == "history")
@@ -279,7 +280,7 @@ void icqconf::loadkeys() {
 	    if (param == "info")
 		k.section = section_info;
 
-	    param = getword(buf);	/* key */
+	    param = getword(buf);       /* key */
 	    if(param == "\\c<space>" || param == "\\C<space>" || param == "^<space>") {
 		k.ctrl = true;
 		k.key = CTRL(' ');
@@ -334,7 +335,7 @@ void icqconf::loadkeys() {
 	    } else
 		k.key = param[0];
 
-	    param = getword(buf);	/* action */
+	    param = getword(buf);       /* action */
 	    if(param == "info") k.action = key_info; else
 	    if(param == "remove") k.action = key_remove; else
 	    if(param == "quit") k.action = key_quit; else
@@ -404,10 +405,12 @@ void icqconf::loadmainconfig() {
 	    if(param == "smtp") setsmtphost(buf); else
 	    if(param == "http_proxy") sethttpproxyhost(buf); else
 	    if(param == "log") makelog = true; else
-	    if(param == "chatmode") initmultiproto(chatmode, buf); else
-	    if(param == "entersends") initmultiproto(entersends, buf); else
-	    if(param == "nonimonline") initmultiproto(nonimonline, buf); else
-	    if(param == "russian" || param == "convert") initmultiproto(cpconvert, buf); else
+	    if(param == "proxy_connect") proxyconnect = true; else
+	    if(param == "proxy_ssl") proxyssl = true; else
+	    if(param == "chatmode") initmultiproto(chatmode, buf, true); else
+	    if(param == "entersends") initmultiproto(entersends, buf, true); else
+	    if(param == "nonimonline") initmultiproto(nonimonline, buf, false); else
+	    if(param == "russian" || param == "convert") initmultiproto(cpconvert, buf, false); else
 	    if(param == "nobidi") setbidi(false); else
 	    if(param == "askaway") askaway = true; else
 	    if(param == "logtimestamps") logtimestamps = true; else
@@ -418,7 +421,7 @@ void icqconf::loadmainconfig() {
 		ptpmin = atoi(getword(buf, "-").c_str());
 		ptpmax = atoi(buf.c_str());
 	    } else {
-		for(pname = icq; pname != protocolname_size; (int) pname += 1) {
+		for(pname = icq; pname != protocolname_size; pname++) {
 		    buf = getprotocolname(pname);
 		    if(param.substr(0, buf.size()) == buf) {
 			im = getourid(pname);
@@ -430,7 +433,7 @@ void icqconf::loadmainconfig() {
 	}
 
 	if(fromcharset.empty() && tocharset.empty())
-	for(pname = icq; pname != protocolname_size; (int) pname += 1) {
+	for(pname = icq; pname != protocolname_size; pname++) {
 	    if(getcpconvert(pname)) {
 		fromcharset = "cp1251";
 		tocharset = "koi8-r";
@@ -438,6 +441,7 @@ void icqconf::loadmainconfig() {
 	    }
 	}
 
+	setproxy();
 	f.close();
     }
 }
@@ -470,25 +474,25 @@ void icqconf::save() {
 	    if(getaskaway()) f << "askaway" << endl;
 
 	    param = "";
-	    for(protocolname pname = icq; pname != protocolname_size; (int) pname += 1)
+	    for(protocolname pname = icq; pname != protocolname_size; pname++)
 		if(getchatmode(pname)) param += (string) " " + conf.getprotocolname(pname);
 	    if(!param.empty())
 		f << "chatmode" << param << endl;
 
 	    param = "";
-	    for(protocolname pname = icq; pname != protocolname_size; (int) pname += 1)
+	    for(protocolname pname = icq; pname != protocolname_size; pname++)
 		if(getentersends(pname)) param += (string) " " + conf.getprotocolname(pname);
 	    if(!param.empty())
 		f << "entersends" << param << endl;
 
 	    param = "";
-	    for(protocolname pname = icq; pname != protocolname_size; (int) pname += 1)
+	    for(protocolname pname = icq; pname != protocolname_size; pname++)
 		if(getnonimonline(pname)) param += (string) " " + conf.getprotocolname(pname);
 	    if(!param.empty())
 		f << "nonimonline" << param << endl;
 
 	    param = "";
-	    for(protocolname pname = icq; pname != protocolname_size; (int) pname += 1)
+	    for(protocolname pname = icq; pname != protocolname_size; pname++)
 		if(getcpconvert(pname)) param += (string) " " + conf.getprotocolname(pname);
 	    if(!param.empty())
 		f << "convert" << param << endl;
@@ -516,6 +520,8 @@ void icqconf::save() {
 	    }
 
 	    if(getmakelog()) f << "log" << endl;
+	    if(getproxyconnect()) f << "proxy_connect" << endl;
+	    if(getproxyssl()) f << "proxy_ssl" << endl;
 
 	    vector<imaccount>::iterator ia;
 	    for(ia = accounts.begin(); ia != accounts.end(); ++ia)
@@ -609,7 +615,7 @@ void icqconf::loadsounds() {
     for(i = 0; i < clist.count; i++) {
 	c = (icqcontact *) clist.at(i);
 
-	for(it = imevent::message; it != imevent::imeventtype_size; (int) it += 1) {
+	for(it = imevent::message; it != imevent::imeventtype_size; it++) {
 	    c->setsound(it, "");
 	}
     }
@@ -687,7 +693,7 @@ void icqconf::loadsounds() {
 		    imcontact ic;
 		    protocolname pname;
 
-		    for(pname = icq; pname != protocolname_size && skey != getprotocolname(pname); (int) pname += 1);
+		    for(pname = icq; pname != protocolname_size && skey != getprotocolname(pname); pname++);
 
 		    if(pname != protocolname_size) {
 			if(suin == "*") {
@@ -1009,7 +1015,7 @@ imstatus icqconf::getstatus(protocolname pname) {
 
     if((ia = a.additional.find("status")) != a.additional.end()) {
 	if(!ia->second.empty()) {
-	    for(st = offline; st != imstatus_size && imstatus2char[st] != ia->second[0]; (int) st += 1);
+	    for(st = offline; st != imstatus_size && imstatus2char[st] != ia->second[0]; st++);
 	    if(st == imstatus_size) st = available;
 	}
     }
@@ -1071,11 +1077,8 @@ void icqconf::commandline(int argc, char **argv) {
 		cw_setbind(bindhost.c_str());
 	    }
 
-	} else if((args == "-P") || (args == "--http-proxy")) {
-	    if(argv[++i]) {
-//                proxyhost = argv[i];
-//                cw_setproxy(proxyhost.c_str());
-	    }
+	} else if((args == "-T") || (args == "--no-xtitles")) {
+	    notitles = true;
 
 	} else if((args == "-s") || (args == "--send")) {
 	    if(argv[++i]) event = argv[i];
@@ -1094,12 +1097,15 @@ void icqconf::commandline(int argc, char **argv) {
 	    if(strlen(argv[i]))
 		st = argv[i][0];
 
+	} else if((args == "-d") || (args == "--debug")) {
+	    debug = true;
+
 	} else if((args == "-v") || (args == "--version")) {
 	    cout << PACKAGE << " " << VERSION << endl
 		<< "Written by Konstantin Klyagin." << endl
 		<< "Built-in protocols are:";
 
-	    for(protocolname pname = icq; pname != protocolname_size; (int) pname += 1)
+	    for(protocolname pname = icq; pname != protocolname_size; pname++)
 		if(gethook(pname).enabled()) cout << " " << conf.getprotocolname(pname);
 
 	    cout << endl << endl
@@ -1152,7 +1158,7 @@ const string &dest, const string &number) const {
     } else {
 	protocolname pname;
 
-	for(pname = icq; pname != protocolname_size; (int) pname += 1) {
+	for(pname = icq; pname != protocolname_size; pname++) {
 	    if(getprotocolname(pname) == proto) {
 		cdest = imcontact(dest, pname);
 		break;
@@ -1189,6 +1195,11 @@ const string &dest, const string &number) const {
     if(ev) {
 	icqcontact *c = new icqcontact(cdest);
 
+	if(access(c->getdirname().c_str(), X_OK)) {
+	    mkdir(c->getdirname().c_str(), S_IRUSR | S_IWUSR | S_IXUSR);
+	    selfsignal(SIGUSR2);
+	}
+
 	if(!access(c->getdirname().c_str(), X_OK)) {
 	    clist.add(c);
 	    em.store(*ev);
@@ -1200,7 +1211,7 @@ const string &dest, const string &number) const {
 
 	    cout << buf << endl;
 	} else {
-	    cout << _("event sending error: the destination contact doesn't exist on your list") << endl;
+	    cout << _("event sending error: error creating directory for the contact") << endl;
 	    exit(1);
 	}
 
@@ -1211,36 +1222,41 @@ const string &dest, const string &number) const {
     }
 }
 
+void icqconf::selfsignal(int signum) const {
+    int rpid;
+    ifstream f(conf.getconfigfname("pid").c_str());
+
+    if(f.is_open()) {
+	f >> rpid;
+	f.close();
+
+	if(rpid > 0) {
+	    kill(rpid, signum);
+	}
+    }
+}
+
 void icqconf::externalstatuschange(char st, const string &proto) const {
     imstatus imst;
     protocolname pname;
-    int rpid;
 
     if(st) {
-	for(pname = icq; pname != protocolname_size; (int) pname += 1)
+	for(pname = icq; pname != protocolname_size; pname++)
 	    if(getprotocolname(pname) == proto)
 		break;
 
-	for(imst = offline; imst != imstatus_size; (int) imst += 1)
+	for(imst = offline; imst != imstatus_size; imst++)
 	    if(imstatus2char[imst] == st)
 		break;
 
 	try {
 	    if(pname != protocolname_size) {
 		if(imst != imstatus_size) {
-		    ifstream f(conf.getconfigfname("pid").c_str());
-		    if(f.is_open()) {
-			f >> rpid;
-			f.close();
+		    string sfname = conf.getconfigfname((string) "status-" + proto);
+		    ofstream sf(sfname.c_str());
+		    if(sf.is_open()) sf << imstatus2char[imst], sf.close();
 
-			if(rpid > 0) {
-			    string sfname = conf.getconfigfname((string) "status-" + proto);
-			    ofstream sf(sfname.c_str());
-			    if(sf.is_open()) sf << imstatus2char[imst], sf.close();
-
-			    kill(rpid, SIGUSR1);
-			}
-		    }
+		    selfsignal(SIGUSR1);
 		} else {
 		    throw _("unknown status character was given");
 		}
@@ -1263,6 +1279,8 @@ void icqconf::usage() const {
     cout << _("  --ascii, -a              use ASCII characters for windows and UI controls") << endl;
     cout << _("  --basedir, -b <path>     set a custom base directory") << endl;
     cout << _("  --bind, -B <host/ip>     bind a custom local IP") << endl;
+    cout << _("  --no-xtitles, -T         disable xterm titles") << endl;
+    cout << _("  --debug, -d              enables debug info logging") << endl;
     cout << _("  --help                   display this stuff") << endl;
     cout << _("  --version, -v            show the program version info") << endl;
 
@@ -1277,25 +1295,50 @@ void icqconf::usage() const {
     cout << endl << _("Report bugs to <centericq-bugs@konst.org.ua>.") << endl;
 }
 
+void icqconf::setproxy() {
+    if(getproxyconnect()) {
+	cw_setproxy(gethttpproxyhost().c_str(), gethttpproxyport(),
+	    gethttpproxyuser().c_str(),
+	    gethttpproxypasswd().c_str());
+
+    } else {
+	cw_setproxy(0, 0, 0, 0);
+
+    }
+}
+
 void icqconf::setmakelog(bool slog) {
     makelog = slog;
 }
 
+void icqconf::setproxyconnect(bool sproxy) {
+    proxyconnect = sproxy;
+    setproxy();
+}
+
+void icqconf::setproxyssl(bool sproxyssl) {
+    proxyssl = sproxyssl;
+}
+
 void icqconf::setgroupmode(icqconf::groupmode amode) {
     fgroupmode = amode;
 }
 
-void icqconf::initmultiproto(bool p[], string buf) {
+void icqconf::initmultiproto(bool p[], string buf, bool excludenochat) {
     string w;
     protocolname pname;
 
-    for(pname = icq; pname != protocolname_size; (int) pname += 1)
+    for(pname = icq; pname != protocolname_size; pname++)
 	p[pname] = buf.empty();
 
     while(!(w = getword(buf)).empty()) {
-	for(pname = icq; pname != protocolname_size; (int) pname += 1) {
+	for(pname = icq; pname != protocolname_size; pname++) {
 	    if(getprotocolname(pname) == w) {
-		p[pname] = true;
+		if(excludenochat) {
+		    p[pname] = !gethook(pname).getCapabs().count(hookcapab::nochat);
+		} else {
+		    p[pname] = true;
+		}
 		break;
 	    }
 	}
@@ -1350,6 +1393,8 @@ void icqconf::sethttpproxyhost(const string &ahttpproxyhost) {
 	    httpproxyport = 8080;
 	}
     }
+
+    setproxy();
 }
 
 string icqconf::getsmtphost() const {
