@@ -125,7 +125,7 @@ class icqconf {
 
 	bool hideoffline, quote, savepwd, antispam, mailcheck,
 	    makelog, fenoughdiskspace, askaway, bidi, logtimestamps,
-	    logonline, emacs;
+	    logonline, emacs, proxyconnect, proxyssl, notitles, debug;
 
 	bool chatmode[protocolname_size];
 	bool cpconvert[protocolname_size];
@@ -151,7 +151,10 @@ class icqconf {
 	    const string &dest, const string &number) const;
 
 	void externalstatuschange(char st, const string &proto) const;
-	void initmultiproto(bool p[], string buf);
+	void initmultiproto(bool p[], string buf, bool excludenochat);
+
+	void selfsignal(int signum) const;
+	void setproxy();
 
     public:
 	icqconf();
@@ -201,6 +204,12 @@ class icqconf {
 	bool getmakelog() const { return makelog; }
 	void setmakelog(bool slog);
 
+	bool getproxyconnect() const { return proxyconnect; }
+	void setproxyconnect(bool proxyconnect);
+
+	bool getproxyssl() const { return proxyssl; }
+	void setproxyssl(bool sproxyssl);
+
 	bool getcpconvert(protocolname pname) const;
 	void setcpconvert(protocolname pname, bool fcpconvert);
 
@@ -278,6 +287,9 @@ class icqconf {
 
 	const char *getconvertfrom(protocolname pname = protocolname_size) const;
 	const char *getconvertto(protocolname pname = protocolname_size) const;
+
+	bool getxtitles() const { return !notitles; }
+	bool getdebug() const { return debug; }
 };
 
 extern icqconf conf;
