@@ -1,7 +1,7 @@
 /*
 *
 * centericq user interface class
-* $Id: icqface.cc,v 1.222 2004/04/11 16:32:28 konst Exp $
+* $Id: icqface.cc,v 1.234 2004/11/12 14:41:45 konst Exp $
 *
 * Copyright (C) 2001-2004 by Konstantin Klyagin <konst@konst.org.ua>
 *
@@ -120,13 +120,13 @@ icqface::~icqface() {
 	for(int i = 0; i < LINES; i++) printf("\n");
     }
 
-#ifdef DEBUG
-    if(flog.is_open()) flog.close();
-#endif
+    if(conf.getdebug())
+    if(flog.is_open())
+	flog.close();
 }
 
 void icqface::init() {
-#ifdef DEBUG
+    if(conf.getdebug())
     if(!flog.is_open()) {
 	time_t logtime = time(0);
 
@@ -134,7 +134,6 @@ void icqface::init() {
 	if(flog.is_open())
 	    flog << endl << "-- centericq debug log started on " << ctime(&logtime);
     }
-#endif
 
     /* Calculate various sizes and coordinates */
 
@@ -190,6 +189,7 @@ void icqface::init() {
 void icqface::done() {
     kt_resize_event = 0;
     workareas.empty();
+    xtermtitlereset();
 }
 
 void icqface::draw() {
@@ -207,7 +207,7 @@ void icqface::showtopbar() {
     protocolname pname;
     icqconf::imaccount ia;
 
-    for(pname = icq; pname != protocolname_size; (int) pname += 1) {
+    for(pname = icq; pname != protocolname_size; pname++) {
 	ia = conf.getourid(pname);
 
 	if(!ia.empty()) {
@@ -268,7 +268,7 @@ int icqface::contextmenu(icqcontact *c) {
 	actnames[ACT_REMOVE]    = _(" Remove channel             del");
 	actnames[ACT_JOIN]      = _(" Join channel");
 	actnames[ACT_LEAVE]     = _(" Leave channel");
-	actnames[ACT_INFO]      = getmenuitem(_("List nichnames"), 25, key_info, section_contact);
+	actnames[ACT_INFO]      = getmenuitem(_("List nicknames"), 25, key_info, section_contact);
 
 	if(lst.inlist(c, csignore))
 	    actnames[ACT_IGNORE]    = getmenuitem(_("UnBlock channel messages"), 25, key_ignore, section_contact);
@@ -448,7 +448,7 @@ icqcontact *icqface::mainloop(int &action) {
 	    g = 0;
 	}
 	
-	if((int) c < 100) c = 0;
+	if((unsigned int) c < 100) c = 0;
 
 	if(i) {
 	    switch(action = extk) {
@@ -675,13 +675,13 @@ void icqface::fillcontactlist() {
 	if(c->getstatus() == offline) {
 	    mcontacts->addleaff(nnode,
 		c->hasevents() ? conf.getcolor(cp_main_highlight) : conf.getprotcolor(c->getdesc().pname),
-		c, "%s%s ", c->hasevents() ? "#" : " ", dnick.c_str());
+		c, "%s%s ", c->hasevents() ? "#" : c->getpostponed().empty() ? " " : ">", dnick.c_str());
 
 	} else {
-	    char *fmt = "%s[%c] %s ";
+	    char *fmt = "%s[%c]%s%s ";
 
-	    if(lst.inlist(c->getdesc(), csvisible)) fmt = "%s<%c> %s "; else
-	    if(lst.inlist(c->getdesc(), csinvisible)) fmt = "%s{%c} %s ";
+	    if(lst.inlist(c->getdesc(), csvisible)) fmt = "%s<%c>%s%s "; else
+	    if(lst.inlist(c->getdesc(), csinvisible)) fmt = "%s{%c}%s%s ";
 
 	    char shortstatus = c->getshortstatus();
 
@@ -692,7 +692,8 @@ void icqface::fillcontactlist() {
 
 	    mcontacts->addleaff(nnode,
 		c->hasevents() ? conf.getcolor(cp_main_highlight) : conf.getprotcolor(c->getdesc().pname),
-		c, fmt, c->hasevents() ? "#" : " ", shortstatus, dnick.c_str());
+		c, fmt, c->hasevents() ? "#" : " ", shortstatus,
+		c->getpostponed().empty() ? " " : ">", dnick.c_str());
 
 	}
     }
@@ -934,7 +935,7 @@ void icqface::infofriends(dialogbox &db, icqcontact *c) {
 }
 
 void icqface::infogeneral(dialogbox &db, icqcontact *c) {
-    string langs, options;
+    string langs, options, buf;
     icqcontact::basicinfo bi = c->getbasicinfo();
     icqcontact::moreinfo mi = c->getmoreinfo();
 
@@ -968,12 +969,37 @@ void icqface::infogeneral(dialogbox &db, icqcontact *c) {
     mainw.write(sizeWArea.x1+14, sizeWArea.y1+12, conf.getcolor(cp_main_text), langs);
     mainw.write(sizeWArea.x1+14, sizeWArea.y1+13, conf.getcolor(cp_main_text), c->getlastip());
 
-    if(c->getstatus() == offline) {
-	time_t ls = c->getlastseen();
+    time_t ls = c->getlastseen();
 
+    if(c->getstatus() == offline) {
 	mainw.write(sizeWArea.x1+2, sizeWArea.y1+14, conf.getcolor(cp_main_highlight), _("Last seen"));
 	mainw.write(sizeWArea.x1+14, sizeWArea.y1+14, conf.getcolor(cp_main_text),
 	    ls ? strdateandtime(ls) : _("Never"));
+
+    } else {
+	int days, hours, minutes, tdiff = timer_current-ls;
+
+	days = (int) (tdiff/86400);
+	hours = (int) ((tdiff-days*86400)/3600);
+	minutes = (int) ((tdiff-days*8600-hours*3600)/60);
+
+	if(days) buf = i2str(days) + " " + _("days");
+
+	if(hours) {
+	    if(!buf.empty()) buf += ", ";
+	    buf += i2str(hours) + " " + _("hours");
+	}
+
+	if(minutes) {
+	    if(!buf.empty()) buf += ", ";
+	    buf += i2str(minutes) + " " + _("min");
+	}
+
+	if(buf.empty())
+	    buf = i2str(tdiff) + " " + _("seconds");
+
+	mainw.write(sizeWArea.x1+2, sizeWArea.y1+14, conf.getcolor(cp_main_highlight), _("Online"));
+	mainw.write(sizeWArea.x1+14, sizeWArea.y1+14, conf.getcolor(cp_main_text), buf);
     }
 }
 
@@ -1180,6 +1206,8 @@ void icqface::userinfo(const imcontact &cinfo, const imcontact &realinfo) {
 	getstatkey(key_user_external_action, section_info).c_str(),
 	getstatkey(key_quit, section_editor).c_str());
 
+    xtermtitle(_("user info for %s"), realinfo.totext().c_str());
+
     db.setwindow(new textwindow(sizeWArea.x1+1, sizeWArea.y1+2, sizeWArea.x2,
 	sizeWArea.y2, conf.getcolor(cp_main_text), TW_NOBORDER));
 
@@ -1287,7 +1315,6 @@ void icqface::userinfo(const imcontact &cinfo, const imcontact &realinfo) {
 
 	    }
 
-
 	    refresh();
 	    showinfo = false;
 	    lastb = b;
@@ -1317,7 +1344,7 @@ void icqface::makeprotocolmenu(verticalmenu &m) {
 	""
     };
 
-    for(ipname = icq; ipname != protocolname_size; (int) ipname += 1) {
+    for(ipname = icq; ipname != protocolname_size; ipname++) {
 	ia = conf.getourid(ipname);
 
 	if(!ia.empty()) {
@@ -1340,7 +1367,7 @@ bool icqface::changestatus(vector<protocolname> &pnames, imstatus &st) {
 
     m.idle = &menuidle;
 
-    for(protcount = 0, pname = icq; pname != protocolname_size; (int) pname += 1) {
+    for(protcount = 0, pname = icq; pname != protocolname_size; pname++) {
 	if(!conf.getourid(pname).empty()) {
 	    protcount++;
 	    onechoice = pname;
@@ -1369,12 +1396,12 @@ bool icqface::changestatus(vector<protocolname> &pnames, imstatus &st) {
     if(r = i) {
 	switch(choice) {
 	    case -1:
-		for(pname = icq; pname != protocolname_size; (int) pname += 1)
+		for(pname = icq; pname != protocolname_size; pname++)
 		    if(!conf.getourid(pname).empty())
 			pnames.push_back(pname);
 		break;
 	    case -2:
-		for(pname = icq; pname != protocolname_size; (int) pname += 1)
+		for(pname = icq; pname != protocolname_size; pname++)
 		    if(!conf.getourid(pname).empty())
 		    if(gethook(pname).getstatus() != offline)
 			pnames.push_back(pname);
@@ -1655,7 +1682,7 @@ void icqface::modelist(contactstatus cs) {
 
     set<protocolname> ps;
     if(cs == csvisible || cs == csinvisible) {
-	for(protocolname pname = icq; pname != protocolname_size; (int) pname += 1)
+	for(protocolname pname = icq; pname != protocolname_size; pname++)
 	    if(gethook(pname).getCapabs().count(hookcapab::visibility))
 		ps.insert(pname);
     }
@@ -1663,7 +1690,7 @@ void icqface::modelist(contactstatus cs) {
     lst.fillmenu(db.getmenu(), cs);
 
     while(db.open(i, b)) {
-	if(!lst.size() && b != 1) continue;
+	if(!db.getmenu()->getcount() && b != 1) continue;
 	if(b != 1) it = lst.menuat(i-1);
 
 	switch(b) {
@@ -1748,7 +1775,7 @@ const set<protocolname> &protos, contactstatus cs) {
 	    imcontact desc = c->getdesc();
 
 	    if(!desc.empty())
-	    if(c->getgroupid() == ig->getid())
+	    if(c->getgroupid() == ig->getid() || conf.getgroupmode() == icqconf::nogroups)
 	    if(protos.empty() || protos.count(desc.pname))
 	    if(!lst.inlist(desc, cs))
 		mlst.push_back(desc);
@@ -1853,16 +1880,12 @@ void icqface::log(const string &atext) {
     int i;
     string text = atext;
 
-#ifdef DEBUG
+    if(conf.getdebug())
     if(flog.is_open())
 	flog << text << endl;
-#endif
 
-    /*
-    *
-    * Add a timestamp if needed
-    *
-    */
+    /*! Add a timestamp if needed
+     */
 
     bool lts, lo;
     conf.getlogoptions(lts, lo);
@@ -1880,7 +1903,7 @@ void icqface::log(const string &atext) {
     while((i = text.find("\n")) != -1) text[i] = ' ';
     while((i = text.find("\r")) != -1) text[i] = ' ';
 
-    while(lastlog.size() > LINES-sizeWArea.y2-2)
+    while((lastlog.size() > LINES-sizeWArea.y2-2) && !lastlog.empty())
 	lastlog.erase(lastlog.begin());
 
     lastlog.push_back(text);
@@ -1954,7 +1977,9 @@ void icqface::quickfind(verticalmenu *multi) {
     verticalmenu *cm = (multi ? multi : &mcontacts->menu);
 
     status(_("QuickSearch: type to find, %s find again, Enter finish"),
-	getstatkey(key_send_message, section_editor).c_str());
+	getstatkey(key_quickfind, section_contact).c_str());
+
+    xtermtitle(_("contact list quick search"));
 
     if(multi) {
 	lx = sizeWArea.x1+2;
@@ -2022,7 +2047,7 @@ void icqface::quickfind(verticalmenu *multi) {
 			    c = (icqcontact *) cm->getref(i);
 			}
 
-			if((int) c > 100) {
+			if((unsigned int) c > 100) {
 			    string current = c->getdispnick();
 			    len = current.size();
 			    if(len > nick.size()) len = nick.size();
@@ -2705,6 +2730,7 @@ vector<eventviewresult> abuttons, bool nobuttons) {
 	    getstatkey(key_show_urls, section_history).c_str(),
 	    getstatkey(key_fullscreen, section_history).c_str(),
 	    getstatkey(key_quit, section_editor).c_str());
+
     }
 
     db.redraw();
@@ -2758,6 +2784,8 @@ void icqface::fullscreenize(const imevent *ev) {
 
     w.write(0, 1, conf.getcolor(cp_main_highlight), buf);
 
+    xtermtitle((string) _("full-screen view") + " " + buf);
+
     tb.setbuf(ev->gettext());
 
     blockmainscreen();
@@ -2813,6 +2841,7 @@ bool icqface::histexec(imevent *&im) {
     bool r, fin;
     int k;
     static string sub;
+    char buf[512];
 
     r = fin = false;
 
@@ -2847,9 +2876,11 @@ bool icqface::histexec(imevent *&im) {
 
 	im = static_cast<imevent *> (mhist.getref(0));
 
-	mainw.writef(sizeWArea.x1+2, sizeWArea.y1, conf.getcolor(cp_main_highlight),
-	    _("History for %s, %d events total"),
-	    im->getcontact().totext().c_str(), mhist.getcount());
+	sprintf(buf, _("History for %s, %d events total"),
+	    im->getcontact().totext().c_str(),
+	    mhist.getcount());
+
+	mainw.write(sizeWArea.x1+2, sizeWArea.y1, conf.getcolor(cp_main_highlight), buf);
 
 	status(_("%s search, %s again, %s cancel"),
 	    getstatkey(key_search, section_history).c_str(),
@@ -2861,6 +2892,7 @@ bool icqface::histexec(imevent *&im) {
 		switch(k) {
 		    case -2:
 			sub = face.inputstr(_("search for: "), sub);
+			if(input.getlastkey() == KEY_ESC) break;
 		    case -3:
 			if(!sub.empty())
 			for(k = mhist.getpos()+1; k < mhist.getcount(); k++) {
@@ -3300,6 +3332,54 @@ void icqface::termresize(void) {
     face.dotermresize = true;
 }
 
+void icqface::relaxedupdate() {
+    fneedupdate = true;
+}
+
+bool icqface::updaterequested() {
+    return fneedupdate;
+}
+
+void icqface::redraw() {
+    if(!mainscreenblock) {
+	done();
+	init();
+	draw();
+	doredraw = fneedupdate = false;
+    } else {
+	doredraw = fneedupdate = true;
+    }
+}
+
+void icqface::xtermtitle(const string &text) {
+    if(conf.getxtitles()) {
+	string term = getenv("TERM") ? getenv("TERM") : "";
+
+	if(term == "xterm" || term == "Eterm" || term == "aterm"
+	|| term == "rxvt" || term == "screen")
+	    cout << "\x1b]1;\x07\x1b]2;" << "centericq" << (text.empty() ? "" : (string) ": " + text) << "\x07" << flush;
+    }
+}
+
+void icqface::xtermtitle(const char *fmt, ...) {
+    if(conf.getxtitles()) {
+	va_list ap;
+	char buf[1024];
+
+	va_start(ap, fmt);
+	vsprintf(buf, fmt, ap);
+	xtermtitle((string) buf);
+	va_end(ap);
+    }
+}
+
+void icqface::xtermtitlereset() {
+    if(conf.getxtitles()) {
+	const char *p = getenv("TERM");
+	if(p) xtermtitle((string) p);
+    }
+}
+
 // ----------------------------------------------------------------------------
 
 icqface::icqprogress::icqprogress() {
@@ -3341,22 +3421,3 @@ void icqface::icqprogress::show(const string &title ) {
 void icqface::icqprogress::hide() {
     w->close();
 }
-
-void icqface::relaxedupdate() {
-    fneedupdate = true;
-}
-
-bool icqface::updaterequested() {
-    return fneedupdate;
-}
-
-void icqface::redraw() {
-    if(!mainscreenblock) {
-	done();
-	init();
-	draw();
-	doredraw = fneedupdate = false;
-    } else {
-	doredraw = fneedupdate = true;
-    }
-}
