@@ -18,6 +18,8 @@ enum protocolname {
     protocolname_size
 };
 
+ENUM_PLUSPLUS(protocolname)
+
 enum imstatus {
     offline = 0,
     available,
@@ -30,6 +32,8 @@ enum imstatus {
     imstatus_size
 };
 
+ENUM_PLUSPLUS(imstatus)
+
 static char imstatus2char[imstatus_size] = {
     '_', 'o', 'i', 'f', 'd', 'c', 'n', 'a'
 };
